'use strict';

const prosemirrorModel = require('prosemirror-model');
const prosemirrorState = require('prosemirror-state');
const prosemirrorView = require('prosemirror-view');

function createVirtualCursor() {
  const key = new prosemirrorState.PluginKey("prosemirror-virtual-cursor");
  return new prosemirrorState.Plugin({
    key,
    props: {
      handleKeyDown: (view, event) => {
        const { selection } = view.state;
        if (event.altKey || event.ctrlKey || event.metaKey || event.shiftKey || event.isComposing || !["ArrowLeft", "ArrowRight"].includes(event.key) || !isTextSelection(selection) || !selection.$cursor)
          return false;
        const $pos = selection.$head;
        const [marksBefore, marksAfter] = getMarksAround($pos);
        const marks = view.state.storedMarks || $pos.marks();
        if (marksBefore && marksAfter && !prosemirrorModel.Mark.sameSet(marksBefore, marksAfter)) {
          if (event.key === "ArrowLeft" && !prosemirrorModel.Mark.sameSet(marksBefore, marks)) {
            view.dispatch(view.state.tr.setStoredMarks(marksBefore));
            return true;
          }
          if (event.key === "ArrowRight" && !prosemirrorModel.Mark.sameSet(marksAfter, marks)) {
            view.dispatch(view.state.tr.setStoredMarks(marksAfter));
            return true;
          }
        }
        if (event.key === "ArrowLeft" && $pos.textOffset === 1) {
          view.dispatch(
            view.state.tr.setSelection(prosemirrorState.TextSelection.create(view.state.doc, $pos.pos - 1)).setStoredMarks($pos.marks())
          );
          return true;
        }
        if (event.key === "ArrowRight" && $pos.textOffset + 1 === $pos.parent.maybeChild($pos.index())?.nodeSize) {
          view.dispatch(
            view.state.tr.setSelection(prosemirrorState.TextSelection.create(view.state.doc, $pos.pos + 1)).setStoredMarks($pos.marks())
          );
          return true;
        }
        return false;
      },
      decorations: (state) => {
        const { selection, doc } = state;
        if (!isTextSelection(selection))
          return null;
        let className = "prosemirror-virtual-cursor";
        let side = 20;
        const $pos = state.selection.$head;
        const [marksBefore, marksAfter] = getMarksAround($pos);
        const marks = state.storedMarks || $pos.marks();
        if (selection.$cursor && marksBefore && marksAfter && marks && !prosemirrorModel.Mark.sameSet(marksBefore, marksAfter)) {
          if (prosemirrorModel.Mark.sameSet(marksBefore, marks)) {
            className += " prosemirror-virtual-cursor-left";
          } else if (prosemirrorModel.Mark.sameSet(marksAfter, marks)) {
            className += " prosemirror-virtual-cursor-right";
            side = -20;
          }
        }
        const deco = prosemirrorView.Decoration.widget(
          $pos.pos,
          () => createCursor(className),
          { side }
        );
        return prosemirrorView.DecorationSet.create(doc, [deco]);
      }
    }
  });
}
function createCursor(className) {
  const span = document.createElement("span");
  span.className = className;
  return span;
}
function getMarksAround($pos) {
  const index = $pos.index();
  const after = $pos.parent.maybeChild(index);
  let before = $pos.textOffset ? after : null;
  if (!before && index > 0)
    before = $pos.parent.maybeChild(index - 1);
  return [before?.marks, after?.marks];
}
function isTextSelection(selection) {
  return selection && typeof selection === "object" && "$cursor" in selection;
}

exports.createVirtualCursor = createVirtualCursor;
